=== modified file 'bzrlib/graph.py'
--- bzrlib/graph.py	2011-12-05 13:04:35 +0000
+++ bzrlib/graph.py	2011-12-05 15:16:52 +0000
@@ -1346,12 +1346,10 @@
         return ('_BreadthFirstSearcher(iterations=%d, %s,'
                 ' seen=%r)' % (self._iterations, search, list(self.seen)))
 
-    def get_result(self):
-        """Get a SearchResult for the current state of this searcher.
+    def get_state(self):
+        """Get the current state of this searcher.
 
-        :return: A SearchResult for this search so far. The SearchResult is
-            static - the search can be advanced and the search result will not
-            be invalidated or altered.
+        :return: Tuple with started keys, excludes and included keys
         """
         if self._returning == 'next':
             # We have to know the current nodes children to be able to list the
@@ -1368,8 +1366,18 @@
             next_query = self._next_query
         excludes = self._stopped_keys.union(next_query)
         included_keys = self.seen.difference(excludes)
+        return self._started_keys, excludes, included_keys
+
+    def _get_result(self):
+        """Get a SearchResult for the current state of this searcher.
+
+        :return: A SearchResult for this search so far. The SearchResult is
+            static - the search can be advanced and the search result will not
+            be invalidated or altered.
+        """
         from bzrlib.vf_search import SearchResult
-        return SearchResult(self._started_keys, excludes, len(included_keys),
+        (started_keys, excludes, included_keys) = self.get_state()
+        return SearchResult(started_keys, excludes, len(included_keys),
             included_keys)
 
     def step(self):

=== modified file 'bzrlib/smart/repository.py'
--- bzrlib/smart/repository.py	2011-12-05 14:12:23 +0000
+++ bzrlib/smart/repository.py	2011-12-05 15:16:52 +0000
@@ -119,15 +119,16 @@
                 except StopIteration:
                     break
                 search.stop_searching_any(exclude_keys.intersection(next_revs))
-            search_result = search.get_result()
-            if (not discard_excess and
-                search_result.get_recipe()[3] != revision_count):
+            (started_keys, excludes, included_keys) = search.get_state()
+            if (not discard_excess and len(included_keys) != revision_count):
                 # we got back a different amount of data than expected, this
                 # gets reported as NoSuchRevision, because less revisions
                 # indicates missing revisions, and more should never happen as
                 # the excludes list considers ghosts and ensures that ghost
                 # filling races are not a problem.
                 return (None, FailedSmartServerResponse(('NoSuchRevision',)))
+            search_result = vf_search.SearchResult(started_keys, excludes,
+                len(included_keys), included_keys)
             return (search_result, None)
         finally:
             repository.unlock()

=== modified file 'bzrlib/tests/test_graph.py'
--- bzrlib/tests/test_graph.py	2011-12-05 13:04:35 +0000
+++ bzrlib/tests/test_graph.py	2011-12-05 15:16:52 +0000
@@ -1029,9 +1029,8 @@
                 search.start_searching(starts)
             if stops is not None:
                 search.stop_searching_any(stops)
-            result = search.get_result()
-            self.assertEqual(recipe, result.get_recipe())
-            self.assertEqual(set(included_keys), result.get_keys())
+            state = search.get_state()
+            self.assertEqual(set(included_keys), state[2])
             self.assertEqual(seen, search.seen)
 
     def test_breadth_first_get_result_excludes_current_pending(self):
@@ -1042,10 +1041,9 @@
             })
         search = graph._make_breadth_first_searcher(['head'])
         # At the start, nothing has been seen, to its all excluded:
-        result = search.get_result()
-        self.assertEqual(('search', set(['head']), set(['head']), 0),
-            result.get_recipe())
-        self.assertEqual(set(), result.get_keys())
+        state = search.get_state()
+        self.assertEqual((set(['head']), set(['head']), set()),
+            state)
         self.assertEqual(set(), search.seen)
         # using next:
         expected = [
@@ -1074,10 +1072,9 @@
         # Starting with nothing and adding a search works:
         search.start_searching(['head'])
         # head has been seen:
-        result = search.get_result()
-        self.assertEqual(('search', set(['head']), set(['child']), 1),
-            result.get_recipe())
-        self.assertEqual(set(['head']), result.get_keys())
+        state = search.get_state()
+        self.assertEqual((set(['head']), set(['child']), set(['head'])),
+            state)
         self.assertEqual(set(['head']), search.seen)
         # using next:
         expected = [
@@ -1234,19 +1231,21 @@
         self.assertSeenAndResult(expected, search, search.next)
         self.assertRaises(StopIteration, search.next)
         self.assertEqual(set(['head', 'ghost', NULL_REVISION]), search.seen)
-        result = search.get_result()
-        self.assertEqual(('search', set(['ghost', 'head']), set(['ghost']), 2),
-            result.get_recipe())
-        self.assertEqual(set(['head', NULL_REVISION]), result.get_keys())
+        state = search.get_state()
+        self.assertEqual(
+            (set(['ghost', 'head']), set(['ghost']),
+                set(['head', NULL_REVISION])),
+            state)
         # using next_with_ghosts:
         search = graph._make_breadth_first_searcher(['head'])
         self.assertSeenAndResult(expected, search, search.next_with_ghosts)
         self.assertRaises(StopIteration, search.next)
         self.assertEqual(set(['head', 'ghost', NULL_REVISION]), search.seen)
-        result = search.get_result()
-        self.assertEqual(('search', set(['ghost', 'head']), set(['ghost']), 2),
-            result.get_recipe())
-        self.assertEqual(set(['head', NULL_REVISION]), result.get_keys())
+        state = search.get_state()
+        self.assertEqual(
+            (set(['ghost', 'head']), set(['ghost']),
+                set(['head', NULL_REVISION])),
+            state)
 
 
 class TestFindUniqueAncestors(TestGraphBase):

=== modified file 'bzrlib/vf_repository.py'
--- bzrlib/vf_repository.py	2011-12-05 14:12:23 +0000
+++ bzrlib/vf_repository.py	2011-12-05 15:16:52 +0000
@@ -2635,7 +2635,9 @@
                 searcher.stop_searching_any(stop_revs)
             if searcher_exhausted:
                 break
-        return searcher.get_result()
+        (started_keys, excludes, included_keys) = searcher.get_state()
+        return vf_search.SearchResult(started_keys, excludes,
+            len(included_keys), included_keys)
 
     @needs_read_lock
     def search_missing_revision_ids(self,

=== modified file 'bzrlib/vf_search.py'
--- bzrlib/vf_search.py	2011-12-05 13:04:35 +0000
+++ bzrlib/vf_search.py	2011-12-05 15:16:52 +0000
@@ -501,9 +501,9 @@
         return [], [], 0
     heads = _find_possible_heads(parent_map, tip_keys, depth)
     s, found_heads = _run_search(parent_map, heads, set(tip_keys))
-    _, start_keys, exclude_keys, key_count = s.get_result().get_recipe()
+    start_keys, exclude_keys, keys = s.get_state()
     if found_heads:
         # Anything in found_heads are redundant start_keys, we hit them while
         # walking, so we can exclude them from the start list.
         start_keys = set(start_keys).difference(found_heads)
-    return start_keys, exclude_keys, key_count
+    return start_keys, exclude_keys, len(keys)

